clear all;
close all;
clc;

set(0,'defaultTextInterpreter','latex');
set(0,'defaultAxesTickLabelInterpreter','latex');

%% add experimental data to path
path1 = 'Experimental_Data_Files';
addpath(genpath(path1))

%% add the robotics toolbox path (if not done manually)
%addpath(genpath('YOUR_PATH/rvctools'));

% % choose the experimental data you want to fit
filenames = {
    'E171003_v0p1_30_load.m';
    'E171003_v0p1_60_load.m';
    'E171003_v0p1_90_load.m';
%     'E171003_v0p15_30_load.m';
%     'E171003_v0p15_60_load.m';
%     'E171003_v0p15_90_load.m';
%     'E171003_v0p2_30_load.m';
%     'E171003_v0p2_60_load.m';
%     'E171003_v0p2_90_load.m';
%     'E171003_v0p2_90_kp150_load.m';
%     'E171004_v0p1_30_load.m';
%     'E171004_v0p1_60_load.m';
%     'E171004_v0p1_90_load.m';
%     'E171004_v0p15_30_load.m';
%     'E171004_v0p15_60_load.m';
%     'E171004_v0p15_90_load.m';
%     'E171004_v0p2_30_load.m';
%     'E171004_v0p2_60_load.m';
%     'E171004_v0p2_90_load.m';
%     'E171004_v0p2_90_kp150_load.m';
    };

%% Load the robot model
run('load_KUKA_model.m');

%% Create the data structure to store the performance results
% For each experiment
perf = zeros(length(filenames),10);
% Globally for all considered experiments
global_perf = 0;

%% Render the fitting process more silent
opt = optimoptions('lsqcurvefit','Display','off');

%% Loop post-impact prediction and fitting process for every experiment
for i = 1:length(filenames)
    %% clear experiment specific data
    % Alternatively the two next lines ensure a drastic variable cleaning
    % clearvars -except i filenames perf global_perf;
    % run('load_KUKA_model.m');
    clearvars -except i filenames perf global_perf opt DD KUKArobot L4 LL...
        ang_screw fp uRolT DEG2RAD L1 L5 MM p DHoffset...
        L2 L6 RAD2DEG ball_radius ring_radius KUKAmodel...
        L3 L7 RR clearance uRHlT;
    
    %% Load the experiment data
    run(filenames{i});
    
    %% Prediction via impact model
    h  = T(2)-T(1); % sampling time (assuming uniform sampling)
    Qd_f = diff(Q) / h; % joint velocity estimated via forward difference
    Qimp = Q(iimp,:); %configuration at impact
    
    % Retrieve frame 7 for the specific used tool (for orientation matrix)
    H_temp = KUKArobot.fkine( Qimp + DHoffset ); % \ls^0 H_7
     
    %  Recall that \ls^T X_DH7 \ls^DH7 V_{0,DH7}
    %  = \ls^T V_{0,T} ( as frame T is fixed wrt to DH7 )
    
    % Compute \ls^C X_{DH 7}
    uCHlT0 = [eye(3), [0; 0; ball_radius]; 0 0 0 1];
    uT0HlT = [H_temp.n H_temp.o H_temp.a, zeros(3,1); 0 0 0 1];
    
    uCXlR = twistTransf(uCHlT0)  * ...  % contact point
        twistTransf(uT0HlT)  * ...      % interial frame orientation
        twistTransf(invHomog(uRHlT));   % from DH frame 7 to frame T (ball center)
    
    Jcontact  = uCXlR * KUKArobot.jacobe(Qimp+DHoffset);
    
    % To be adapted to the considered normal direction
    JN = Jcontact(3,:);   % normal impact part of the Jacobian
    
    vm = Qd_f(iimp,:).'; % ante impact velocity
    
    M_temp  = reshape(M(iimp,:),7,7);
    
    lambda  = - inv( JN * inv(M_temp) * JN.') * JN * vm; % normal impact
    Qd_plus = transpose(vm + inv(M_temp) * JN.' * lambda); % post-impact joint velocities
    xyzd_plus = (Jcontact(1:3,:) * Qd_plus.').'; % cartesian post-impact velocity
    xyzd_minus = Jcontact(1:3,:) * vm;
    
    %% Fitting
    % To be adapted to the considered directions of interest
    nn = [3 1];  % relevant directions, z-direction and x-direction
    
    NN = 150; % number of data points considered for the fitting
    N = iimp:iimp+NN;
    Nbis = iimp-50:iimp+150; % for plotting purposes only
    
    % fit every direction
    for j = 1:length(nn)
        
        % collect cartesian velocity through expdata
        x_i = zeros(1,length(N));
        for ii = 1:length(N)
            Qi = Q(N(ii),:);
            Qdi = Qd_f(N(ii),:);
            xyzabcd = uCXlR * KUKArobot.jacobe(Qi+DHoffset)*Qdi';
            x_i(ii) = xyzabcd(nn(j));
        end
        
        % actual fitting
        ydata = x_i'; %data to be fitted
        xdata = T(N)-T(iimp); %shift time to zero
        
        %%%%%%%%%%%%%%%%%%%%%%%%%
        % for plotting purposes %
        x_ibis = zeros(1,length(Nbis));
        for ii = 1:length(Nbis)
            Qi = Q(Nbis(ii),:);
            Qdi = Qd_f(Nbis(ii),:);
            xyzabcd = uCXlR * KUKArobot.jacobe(Qi+DHoffset)*Qdi';
            x_ibis(ii) = xyzabcd(nn(j));
        end
        
        ydatabis = x_ibis';
        xdatabis = T(Nbis)-T(iimp);
        %%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        % initial estimations parameters
        [xu,ixu] = max(ydata);
        [xl,ixl] = min(ydata);
        xr = (xu-xl); % Range of x
        % Manually defined based on visual inspection of the signals
        per = 0.06;
        xdm = (ydata(end)-ydata(1))/(NN*h); % average slope
        omega = 1/per;
        gamma = 0.01;
        
        if(j == 1) % in order to keep the same mode for next fitting directions
            x10 = [xr;gamma; omega;  0.0; xdm]; % vector with initial guesses
            fun1 = @(x1,xdata)x1(1)*exp(-x1(2)*xdata).*cos(x1(3)*2*pi*xdata+x1(4))+(ydata(1)-x1(1)*cos(x1(4)))+x1(5)*xdata; % fitting function
            
            % lsq fitting
            x1 = lsqcurvefit(fun1,x10,xdata,ydata,[],[],opt);  % results fitting
            fit1 = x1(1)*exp(-x1(2)*xdata).*cos(x1(3)*2*pi*xdata+x1(4))+(ydata(1)-x1(1)*cos(x1(4)))+x1(5)*xdata; % apply results to data
            lin = (ydata(1)-x1(1)*cos(x1(4)))+x1(5)*xdata; % linear term
            my_freq = x1(3);
            my_gamma = x1(2);
        else
            x10 = [xr; 0.0; xdm]; % vector with initial guesses
            fun1 = @(x1,xdata)x1(1)*exp(-my_gamma*xdata).*cos(my_freq*2*pi*xdata+x1(2))+(ydata(1)-x1(1)*cos(x1(2)))+x1(3)*xdata; % fitting function
            
            % lsq fitting
            x1 = lsqcurvefit(fun1,x10,xdata,ydata,[],[],opt);  % results fitting
            fit1 = x1(1)*exp(-my_gamma*xdata).*cos(my_freq*2*pi*xdata+x1(2))+(ydata(1)-x1(1)*cos(x1(2)))+x1(3)*xdata; % apply results to data
            lin = (ydata(1)-x1(1)*cos(x1(2)))+x1(3)*xdata; % linear term
        end
        
        %% plots
        f = figure(1);
        set(f,'PaperOrientation','landscape');
        set(f,'PaperUnits','normalized');
        set(f,'PaperPosition', [0 0 1 1]);
        subplot(length(filenames),2,2*(i-1)+j)
        plot(xdatabis,ydatabis,'LineWidth',1.5)         %expdata
        hold on
        plot(xdata,fit1,'k--','LineWidth',1.5)    %fit
        plot(xdata,lin,'r.-','LineWidth',1.5)     %linear term fit
        plot(0,xyzd_plus(nn(j)),'-s','MarkerSize',8,...% post-impact Cartesian velocity
            'MarkerEdgeColor','red',...
            'MarkerFaceColor',[1 .6 .6])
        grid on;
        grid minor;
        xlabel('time [s]','Interpreter','latex','FontSize',14)
        xlim([-0.05 0.150]);
        my_axis = ['x' 'y' 'z'];
        my_ylabel = strcat('$\dot{',my_axis(nn(j)),'}$ [$m/s$]');
        ylabel(my_ylabel,'Interpreter','latex','FontSize',14);
        my_title = strcat(filenames{i}(length(filenames{i})-8:length(filenames{i})-7),' deg.');%,' v',my_axis(nn(j)),'- = ',num2str(xyzd_minus(nn(j))),' v',my_axis(nn(j)),'+rbm= ',num2str(xyzd_plus(nn(j))),' v',my_axis(nn(j)),'+fit= ',num2str(lin(1)));
        title(my_title,'Interpreter','none','FontSize',12,'FontWeight','normal');
        
        %% Synthetic results
        % Format : 
        % (1) Considered cartesian axis
        % (2) Ante impact cartesian velocity
        % (3) Post impact cartesian velocity estimation using impact map 
        % (4) Post impact cartesian velocity estimation using fitting
        % (5) Estimation error
        perf(i,5*j-4:5*j) = [nn(j) xyzd_minus(nn(j)) xyzd_plus(nn(j)) lin(1) nu_indicator(xyzd_plus(nn(j)),lin(1))];
        global_perf = global_perf + nu_indicator(xyzd_plus(nn(j)),lin(1));
    end
end

%% Finalize the plot
% legend('Actual','Fitting function $f_{tot}$','Linear term fitting','Impact law pred','FontSize',10,'Location','Best','Interpreter','latex')
legend('Actual','Fitting function f_{tot}','Linear term fitting','Impact law pred')

%% Display the synthetic results
perf_round = round(perf,3);
disp(' ')
disp('Fitting result table')
disp(perf_round)

function nu = nu_indicator(v1,v2)
nu = abs(v1-v2);
end

function nupc = nupc_indicator(v1,v2)
    nupc = 100*nu_indicator(v1,v2)/abs(v2);
end

