% This function create the KUKArobot model and
% define kinematic quantities for the other MATLAB scripts
%
% Load Robotic Toolbox' path is set

% if ~exist('rtbdemo')
%   error('Peter Coorke''s Robotics Toolbox is not accessible')  
% end

startup_rvc; % set up the robotics toolbox 

global KUKAmodel; % used to check, in other scripts, if this script has been loaded

% DH table for the KUKA LWB IV+ robot 
%
% th   d   a   alpha
% ---------------------
% q1   D   0    pi/2
% q2   0   0   -pi/2
% q3   0   0   -pi/2
% q4   L   0    pi/2
% q5   0   0    pi/2
% q6   M   0   -pi/2
% q7   0   0      0
%
% L = 0.40
% M = 0.39

LL = 0.40;
MM = 0.39;

DD = 0.102+0.2085+0.020; % UPMC KUKA ROBOT (typically zero)
                         % See URDF file 
                         % https://github.com/kuka-isir/rtt_lwr/blob/rtt_lwr-2.0/lwr_description/urdf/lwr.urdf.xacro
                         % and (for 0.020)
                         % https://github.com/kuka-isir/rtt_lwr/blob/rtt_lwr-2.0/lwr_description/robots/kuka_lwr.urdf.xacro
                         
RR  = 0.078;  % distance between the wrist center and 
              % the mounting ring
                         

              
L1 = Link('d', DD, 'a', 0, 'alpha',  pi/2);
L2 = Link('d',  0, 'a', 0, 'alpha', -pi/2);
L3 = Link('d', LL, 'a', 0, 'alpha', -pi/2);
L4 = Link('d',  0, 'a', 0, 'alpha',  pi/2);
L5 = Link('d', MM, 'a', 0, 'alpha',  pi/2);
L6 = Link('d',  0, 'a', 0, 'alpha', -pi/2);
L7 = Link('d', RR, 'a', 0, 'alpha',     0);

KUKArobot = SerialLink([L1 L2 L3 L4 L5 L6 L7], 'name', 'KUKA LWB IV+');
% KUKArobotpc2 = SerialLink([L1 L2 L3 L4 L5 L6], 'name', 'KUKA LWB IV+')
% UMPC specific KUKA robot parameters, used during the data acquisition

% matching the zero position for q1 (x axis is rotated 120 degrees)
DHoffset = [2*pi/3 0 0 0 0 0 0];

DEG2RAD = pi/180;
RAD2DEG = 180/pi;

ring_radius = 0.025; % from KUKA datasheet (8 screws, 45 deg apart)                    
                    
ball_radius = 0.025; % radius of the impacting ball, connected with a screw 
                     % to the mounting ring

clearance = 0.001;   % distance between the ball and mounting ring               
              
% T   - Ball frame
% R   - Ring frame (DH frame 7)
 
ang_screw = 0*DEG2RAD; % -10 DEG FOUND BY VISUAL INSPECTION
% disp('ATTENTION: -10 deg emphirical correction applied !!')

% R - Ball frame wrt Ring frame (AKA UPMC frame 7) 
uRolT  = [ring_radius*cos(ang_screw);ring_radius*sin(ang_screw); ball_radius + clearance];
uRHlT  = [eye(3), uRolT; 0 0 0, 1]; 





