close all
clear all

set(0,'defaultTextInterpreter','latex')
set(0,'defaultAxesTickLabelInterpreter','latex')


% Make sure to run first 
% sim_Flex_Compliant_modelA  -> FS_results.mat
% sim_Flex_Hard_modelB       -> FH_results.mat
% sim_Rigid_Hard_modelC      -> RH_results.mat 

load FS_results
load FH_results
load RH_results

zoom_min = 0.11; %[s];  % hard coded visualization bounds
zoom_max = 0.23; %[s];

FS_zoom_t_index = (FS.t > zoom_min) & (FS.t < zoom_max);
FH_zoom_t_index = (FH.t > zoom_min) & (FH.t < zoom_max);
RH_zoom_t_index = (RH.t > zoom_min) & (RH.t < zoom_max);


figure(1)
FSIZE = 10; % font size
set(0, 'DefaultLineLineWidth', 1); % line width

subplot(231), hold on  % mass 1 position
  plot(FS.t, FS.x1)
  plot(FH.t, FH.x1, 'k--')
  plot(RH.t, RH.x, 'r-.')
  ylabel('$m_1$ pos. [m]')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)
subplot(232), hold on  % mass 1 velocity
  plot(FS.t,  FS.x1d)
  plot(FH.t,  FH.x1d,'k--')
  plot(RH.t, RH.xd, 'r-.')  
  ylabel('$m_1$ vel. [m/s]')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)
subplot(233), hold on % mass 1 velocity zoomed in
  plot(FS.t(FS_zoom_t_index), FS.x1d(FS_zoom_t_index))
  plot(FH.t(FH_zoom_t_index), FH.x1d(FH_zoom_t_index),'k--')
  plot(RH.t(RH_zoom_t_index), RH.xd(RH_zoom_t_index), 'r-.')
  T = FS.t(FS_zoom_t_index); 
  axis([T(1) T(end) -3.3 -0.3])   % hard coded visualization bounds
  ylabel('$m_1$ vel. [m/s] (zoom)')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)

subplot(234), hold on % mass 2 position
  plot(FS.t, FS.x2)
  plot(FH.t, FH.x2, 'k--')
  plot(RH.t, RH.x, 'r-.')
  ylabel('$m_2$ pos. [m]')
  xlabel('time [s]')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)
subplot(235), hold on % mass 2 velocity
  plot(FS.t, FS.x2d)
  plot(FH.t,  FH.x2d,'k--')
  plot(RH.t, RH.xd, 'r-.')
  ylabel('$m_2$ vel. [m/s]')
  xlabel('time [s]')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)
subplot(236), hold on % mass 2 velocity zoomed in
  plot(FS.t(FS_zoom_t_index), FS.x2d(FS_zoom_t_index))
  plot(FH.t(FH_zoom_t_index), FH.x2d(FH_zoom_t_index),'k--')
  plot(RH.t(RH_zoom_t_index), RH.xd(RH_zoom_t_index), 'r-.')  
  % T = FS.t(FS_zoom_t_index); 
  axis([T(1) T(end) -3.3 -0.3])   % hard coded visualization bounds
  ylabel('$m_2$ vel. [m/s] (zoom)')
  xlabel('time [s]')
  grid on, zoom on
  set(gca,'FontSize',FSIZE)